<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Harry Fuecks <hfuecks@phppatterns.com>                      |
// +----------------------------------------------------------------------+
//
// $Id: Decorator.php,v 1.2 2003/10/30 00:02:07 quipo Exp $
//
/**
 * @package Calendar
 * @version $Id: Decorator.php,v 1.2 2003/10/30 00:02:07 quipo Exp $
 */
/**
 * Decorates any calendar class. Create a subclass of this class for your
 * own "decoration". Used for "selections"
 * <code>
 * class DayDecorator extends Calendar_Decorator {
 *     function thisDay($asTs = false) {
.*         $day = parent::thisDay(TRUE);
.*         return date('D',$day);
 *     }
 * }
 * $Day = & new Calendar_Day(2003,10,25);
 * $DayDecorator = & new DayDecorator($Day);
 * echo ( $DayDecorator->thisDay() ); // Outputs "Sat"
 * </code>
 * @abstract
 * @package Calendar
 */
class Calendar_Decorator
{
    /**
     * Subclass of Calendar being decorated
     * @var object
     * @access private
     */
    var $calendar;

    /**
     * Constructs the Calendar_Decorator
     * @param object subclass to Calendar to decorate
     */
    function Calendar_Decorator(& $calendar)
    {
        $this->calendar = & $calendar;
    }

    /**
     * Defines the calendar by a Unix timestamp, replacing values
     * passed to the constructor
     * @param int Unix timestamp
     * @return void
     * @access public
     */
    function setTimestamp($ts)
    {
        $this->calendar->setTimestamp($ts);
    }

    /**
     * Returns a timestamp from the current date / time values. Format of
     * timestamp depends on Calendar_Engine implementation being used
     * @return int timestamp
     * @access public
     */
    function getTimestamp()
    {
        return $this->calendar->getTimeStamp();
    }

    /**
     * Defines calendar object as selected (e.g. for today)
     * @param boolean state whether Calendar subclass
     * @return void
     * @access public
     */
    function setSelected($state = true)
    {
        $this->calendar->setSelected($state = true);
    }

    /**
     * True if the calendar subclass object is selected (e.g. today)
     * @return boolean
     * @access public
     */
    function isSelected()
    {
        return $this->calendar->isSelected();
    }

    /**
     * Defines Day object as first in a week
     * Only used by Calendar_Month_Weekdays::build()
     * @param boolean state
     * @return void
     * @access private
     */
    function setFirst ($state = true)
    {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            $this->calendar->setFirst($state);
        }
    }

    /**
     * Defines Day object as last in a week
     * Used only following Calendar_Month_Weekdays::build()
     * @param boolean state
     * @return void
     * @access private
     */
    function setLast($state = true)
    {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            $this->calendar->setLast($state);
        }
    }

    /**
     * Returns true if Day object is first in a Week
     * Only relevant when Day is created by Calendar_Month_Weekdays::build()
     * @return boolean
     * @access public
     */
    function isFirst() {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            return $this->calendar->isFirst();
        }
    }

    /**
     * Returns true if Day object is last in a Week
     * Only relevant when Day is created by Calendar_Month_Weekdays::build()
     * @return boolean
     * @access public
     */
    function isLast()
    {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            return $this->calendar->isLast();
        }
    }

    /**
     * Defines Day object as empty
     * Only used by Calendar_Month_Weekdays::build()
     * @param boolean state
     * @return void
     * @access private
     */
    function setEmpty ($state = true)
    {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            $this->calendar->setEmpty($start);
        }
    }

    /**
     * @return boolean
     * @access public
     */
    function isEmpty()
    {
        $class = strtolower(get_class($this->calendar));
        if ($class == 'calendar_day') {
            return $this->calendar->isEmpty();
        }
    }

    /**
     * Build the children
     * @param array containing Calendar objects to select (optional)
     * @return boolean
     * @access public
     * @abstract
     */
    function build($sDates = array())
    {
        $this->calendar->build($sDates);
    }

    /**
     * Iterator method for fetching child Calendar subclass objects
     * (e.g. a minute from an hour object). On reaching the end of
     * the collection, returns false and resets the collection for
     * further iteratations.
     * @return mixed either an object subclass of Calendar or false
     * @access public
     */
    function fetch()
    {
        return $this->calendar->fetch();
    }

    /**
     * Fetches all child from the current collection of children
     * @return array
     * @access public
     */
    function fetchAll()
    {
        return $this->calendar->fetchAll();
    }

    /**
     * Get the number Calendar subclass objects stored in the internal
     * collection.
     * @return int
     * @access public
     */
    function size()
    {
        return $this->calendar->size();
    }

    /**
     * Determine whether this date is valid, with the bounds determined by
     * the Calendar_Engine. The call is passed on to
     * Calendar_Validator::isValid
     * @return boolean
     * @access public
     */
    function isValid()
    {
        return $this->calendar->isValid();
    }

    /**
     * Returns an instance of Calendar_Validator
     * @return Calendar_Validator
     * @access public
     */
    function & getValidator()
    {
        return $this->calendar->getValidator();
    }

    /**
     * Returns a reference to the current Calendar_Engine being used. Useful
     * for Calendar_Table_Helper and Caledar_Validator
     * @return object implementing Calendar_Engine_Inteface
     * @access private
     */
    function & getEngine()
    {
        return $this->calendar->getEngine();
    }

    /**
     * Returns the value for the previous year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2002 or timestamp
     * @access public
     */
    function prevYear($asTs = false)
    {
        return $this->calendar->prevYear($asTs);
    }

    /**
     * Returns the value for this year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2003 or timestamp
     * @access public
     */
    function thisYear($asTs = false)
    {
        return $this->calendar->thisYear($asTs);
    }

    /**
     * Returns the value for next year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2004 or timestamp
     * @access public
     */
    function nextYear($asTs = false)
    {
        return $this->calendar->nextYear($asTs);
    }

    /**
     * Returns the value for the previous month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 4 or Unix timestamp
     * @access public
      */
    function prevMonth($asTs = false)
    {
        return $this->calendar->prevMonth($asTs);
    }

    /**
     * Returns the value for this month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 5 or timestamp
     * @access public
     */
    function thisMonth($asTs = false)
    {
        return $this->calendar->thisMonth($asTs);
    }

    /**
     * Returns the value for next month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 6 or timestamp
     * @access public
     */
    function nextMonth($asTs = false)
    {
        return $this->calendar->nextMonth($asTs);
    }

    /**
     * Returns the value for the previous week
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 4 or Unix timestamp
     * @access public
      */
    function prevWeek($asTs = false)
    {
        if (strtolower(get_class($this->calendar)) != 'calendar_week') {
            require_once 'PEAR.php';
            PEAR::raiseError(
                'Cannot call prevWeek on Calendar object of type: '.
                get_class($this->calendar), 133, PEAR_ERROR_TRIGGER,
                E_USER_NOTICE, 'Calendar_Decorator::prevWeek()');
            return false;
        } else {
            return $this->calendar->prevWeek($asTs);
        }
    }

    /**
     * Returns the value for this week
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 5 or timestamp
     * @access public
     */
    function thisWeek($asTs = false)
    {
        if (strtolower(get_class($this->calendar)) != 'calendar_week') {
            require_once 'PEAR.php';
            PEAR::raiseError(
                'Cannot call thisWeek on Calendar object of type: '.
                get_class($this->calendar), 133, PEAR_ERROR_TRIGGER,
                E_USER_NOTICE, 'Calendar_Decorator::thisWeek()');
            return false;
        } else {
            return $this->calendar->thisWeek($asTs);
        }
    }

    /**
     * Returns the value for next week
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 6 or timestamp
     * @access public
     */
    function nextWeek($asTs = false)
    {
        if (strtolower(get_class($this->calendar)) != 'calendar_week') {
            require_once 'PEAR.php';
            PEAR::raiseError(
                'Cannot call nextWeek on Calendar object of type: '.
                get_class($this->calendar), 133, PEAR_ERROR_TRIGGER,
                E_USER_NOTICE, 'Calendar_Decorator::nextWeek()');
            return false;
        } else {
            return $this->calendar->nextWeek($asTs);
        }
    }

    /**
     * Returns the value for the previous day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 10 or timestamp
     * @access public
     */
    function prevDay($asTs = false) {
        return $this->calendar->prevDay($asTs);
    }

    /**
     * Returns the value for this day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 11 or timestamp
     * @access public
     */
    function thisDay($asTs=false)
    {
        return $this->calendar->thisDay($asTs);
    }

    /**
     * Returns the value for the next day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 12 or timestamp
     * @access public
     */
    function nextDay($asTs=false)
    {
        return $this->calendar->nextDay($asTs);
    }

    /**
     * Returns the value for the previous hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 13 or timestamp
     * @access public
     */
    function prevHour($asTs=false)
    {
        return $this->calendar->prevHour($asTs);
    }

    /**
     * Returns the value for this hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function thisHour($asTs = false)
    {
        return $this->calendar->thisHour($asTs);
    }

    /**
     * Returns the value for the next hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function nextHour($asTs = false)
    {
        return $this->calendar->nextHour($asTs);
    }

    /**
     * Returns the value for the previous minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 23 or timestamp
     * @access public
     */
    function prevMinute($asTs = false)
    {
        return $this->calendar->prevMinute($asTs);
    }

    /**
     * Returns the value for this minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 24 or timestamp
     * @access public
     */
    function thisMinute($asTs = false)
    {
        return $this->calendar->thisMinute($asTs);
    }

    /**
     * Returns the value for the next minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 25 or timestamp
     * @access public
     */
    function nextMinute($asTs = false)
    {
        return $this->calendar->nextMinute($asTs);
    }

    /**
     * Returns the value for the previous second
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 43 or timestamp
     * @access public
     */
    function prevSecond($asTs = false)
    {
        return $this->calendar->prevSecond($asTs);
    }

    /**
     * Returns the value for this second
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 44 or timestamp
     * @access public
     */
    function thisSecond($asTs = false)
    {
        return $this->calendar->thisSecond($asTs);
    }

    /**
     * Returns the value for the next second
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 45 or timestamp
     * @access public
     */
    function nextSecond($asTs = false)
    {
        return $this->calendar->nextSecond($asTs);
    }
}
?>