<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Harry Fuecks <hfuecks@phppatterns.com>                      |
// +----------------------------------------------------------------------+
//
// $Id: Week.php,v 1.5 2003/10/30 14:11:24 harryf Exp $
//
/**
 * @package Calendar
 * @version $Id: Week.php,v 1.5 2003/10/30 14:11:24 harryf Exp $
 */

/**
* Allows Calendar include path to be redefined
*/
if (!defined('CALENDAR_ROOT')) {
    define('CALENDAR_ROOT', 'Calendar'.DIRECTORY_SEPARATOR);
}

/**
 * Load Calendar base class
 */
require_once CALENDAR_ROOT.'Calendar.php';

/**
 * Represents a Week and builds Days in tabular format<br>
 * <code>
 * require_once('Calendar/Week.php');
 * $Week = & new Calendar_Week(2003,10,1); Oct 2003, 1st tabular week
 * echo '<tr>';
 * while ($Day = & $Week->fetch()) {
 *     if ($Day->isEmpty()) {
 *         echo '<td>&nbsp;</td>';
 *     } else {
 *         echo '<td>'.$Day->thisDay().'</td>';
 *      }
 * }
 * echo '</tr>';
 * </code>
 * @package Calendar
 * @access public
 */
class Calendar_Week extends Calendar
{
    /**
     * Instance of Calendar_Table_Helper
     * @var Calendar_Table_Helper
     * @access private
     */
    var $tableHelper;

    /**
     * Stores the numeric value of this week relative to others in the month
     * @access private
     * @var object
     */
    var $thisWeek;

    /**
     * Constructs Week
     * @param int year e.g. 2003
     * @param int month e.g. 5
     * @param int tabular week of month e.g. 3
     * @param int (optional) first day of week (e.g. 0 for Sunday, 2 for Tuesday etc.)
     * @access public
     */
    function Calendar_Week($y, $m, $w, $firstDay=false)
    {
        require_once CALENDAR_ROOT.'Table'.DIRECTORY_SEPARATOR.'Helper.php';
        Calendar::Calendar($y, $m);
        $this->thisWeek = $w;
        $this->tableHelper = & new Calendar_Table_Helper($this, $firstDay);
    }

    /**
     * Builds Calendar_Day objects for this Week
     * @param array (optional) Calendar_Day objects representing selected dates
     * @return boolean
     * @access public
     */
    function build($sDates = array())
    {
        require_once CALENDAR_ROOT.'Day.php';
        $numWeeks = $this->tableHelper->getNumWeeks();
        if ($this->thisWeek == 1) {
            $this->buildFirstWeek();
        } else if ($this->thisWeek == $numWeeks) {
            $this->buildLastWeek();
        } else {
            $this->buildWeek();
        }
        if (count($sDates) > 0) {
            $this->setSelection($sDates);
        }
        return true;
    }

    /**
     * Builds the first tabular week of the month
     * @return void
     * @access private
     */
    function buildFirstWeek()
    {
        $eBefore = $this->tableHelper->getEmptyDaysBefore();
        for ($i=1; $i < $eBefore; $i++) {
            $this->children[$i] = new Calendar_Day($this->year, $this->month, $i-$eBefore+1);
            $this->children[$i]->setEmpty();
        }
        $end = $this->cE->getDaysInWeek();
        if ( $eBefore == 0 )
            $eBefore = 1;
        $day = 1;
        for ($i = $eBefore; $i <= $end; $i++) {
            $this->children[$i] = new Calendar_Day($this->year, $this->month, $day);
            $day++;
        }
    }

    /**
     * Builds the last tabular week of the month
     * @return void
     * @access private
     */
    function buildLastWeek()
    {
        $eBefore = $this->tableHelper->getEmptyDaysBefore();
        $day = ((($this->thisWeek-1) * $this->cE->getDaysInWeek())+1) - $eBefore;
        $end = $this->cE->getDaysInWeek();
        $eAfter = $this->tableHelper->getEmptyDaysAfterOffset();
        for ($i = 1; $i <= $end; $i++, $day++) {
            $this->children[$i] = new Calendar_Day($this->year, $this->month, $day);
            if ($i > $eAfter) {
                $this->children[$i]->setEmpty();
            }
        }
    }

    /**
     * Use to build any tabular weeks apart from the first and last
     * @return void
     * @access private
     */
    function buildWeek()
    {
        $eBefore = $this->tableHelper->getEmptyDaysBefore();
        $day = ((($this->thisWeek-1) * $this->cE->getDaysInWeek())+1) - $eBefore;
        $end = $this->cE->getDaysInWeek();
        for ($i=1; $i <= $end; $i++) {
            $this->children[$i] = new Calendar_Day($this->year, $this->month, $day);
            $day++;
        }
    }

    /**
     * Called from build()
     * @param array
     * @return void
     * @access private
     */
    function setSelection($sDates)
    {
        $offset = ($this->thisWeek-1) * $this->cE->getDaysInWeek();
        $eBefore = $this->tableHelper->getEmptyDaysBefore();
        foreach ($sDates as $sDate) {
            if ($this->thisYear() == $sDate->thisYear()
                && $this->thisMonth() == $sDate->thisMonth())
            {
                $key = (int)($sDate->thisDay() - $offset + $eBefore);
                if (isset($this->children[$key])) {
                    $sDate->setSelected();
                    $this->children[$key] = $sDate;
                }
           }
        }
    }

    /**
     * Gets the numeric value of the previous week in the month
     * or NULL if this is the first week of the month
     * @return int
     * @access public
     */
    function prevWeek($asTs = FALSE)
    {
        $prevWeek = $this->thisWeek - 1;
        if ( $prevWeek > 0 ) {
            if ( !$asTs ) {
                return $prevWeek;
            } else {
                require_once CALENDAR_ROOT.'Month/Weeks.php';
                $Month = new Calendar_Month_Weeks($this->year,$this->month);
                $Month->build();
                $weeks = $Month->fetchAll();
                return $weeks[$prevWeek]->getTimeStamp();
            }
        } else {
            require_once CALENDAR_ROOT.'Month/Weeks.php';
            $monthStamp = $this->prevMonth(true);
            $Month = new Calendar_Month_Weeks($this->cE->stampToYear($monthStamp),
                                              $this->cE->stampToMonth($monthStamp));
            $Month->build();
            $weeks = $Month->fetchAll();
            $Week = $weeks[$Month->size()];
            if ( !$asTs ) {
                return $Week->thisWeek();
            } else {
                return $Week->getTimeStamp();
            }
        }
    }

    /**
     * Gets the numeric value of the week in the month
     * @return int
     * @access public
     */
    function thisWeek($asTs = FALSE)
    {
        if ( !$asTs ) {
            return $this->thisWeek;
        } else {
            return $this->getTimeStamp();
        }
    }

    /**
     * Gets the numeric value of the next week in the month
     * or NULL if this is the last week of the month
     * @return int
     * @access public
     */
    function nextWeek($asTs = FALSE)
    {
        $nextWeek = $this->thisWeek + 1;
        if ( $nextWeek <= $this->tableHelper->getNumWeeks() ) {
            if ( !$asTs ) {
                return $nextWeek;
            } else {
                require_once CALENDAR_ROOT.'Month/Weeks.php';
                $Month = new Calendar_Month_Weeks($this->year,$this->month);
                $Month->build();
                $weeks = $Month->fetchAll();
                return $weeks[$nextWeek]->getTimeStamp();
            }
        } else {
            require_once CALENDAR_ROOT.'Month/Weeks.php';
            $monthStamp = $this->nextMonth(true);
            $Month = new Calendar_Month_Weeks($this->cE->stampToYear($monthStamp),
                                              $this->cE->stampToMonth($monthStamp));
            $Month->build();
            $Week = $Month->fetch();
            if ( !$asTs ) {
                return $Week->thisWeek();
            } else {
                return $Week->getTimeStamp();
            }
        }
    }

    /**
     * Returns the value for the previous day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 10 or timestamp
     * @access public
     */
    function prevDay($asTs = false)
    {
        $this->findFirstDay();
        return Calendar::prevDay($asTs);
    }

    /**
     * Returns the value for this day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 11 or timestamp
     * @access public
     */
    function thisDay($asTs = false)
    {
        $this->findFirstDay();
        return Calendar::thisDay($asTs);
    }

    /**
     * Returns the value for the next day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 12 or timestamp
     * @access public
     */
    function nextDay($asTs = false)
    {
        $this->findFirstDay();
        return Calendar::nextDay($asTs);
    }

    /**
     * Returns a timestamp from the current date / time values<br />
     * <b>Note:</b> overrides parent method as Week must determine
     * the first day in the collection.<br />
     * <b>Note:</b> calling this method also
     * calls the build() method, if it has not already called.
     * @return int Unix timestamp
     * @access public
     */
    function getTimestamp()
    {
        $this->findFirstDay();
        return Calendar::getTimeStamp();
    }

    /**
     * Returns the instance of Calendar_Table_Helper.
     * Called from Calendar_Validator::isValidWeek
     * @return Calendar_Table_Helper
     * @access protected
     */
    function & getHelper()
    {
        return $this->tableHelper;
    }

    /**
     * Makes sure theres a value for $this->day
     * @return void
     * @access private
     */
    function findFirstDay()
    {
        if (!count($this->children) > 0) {
            $this->build();
            foreach ( $this->children as $Day ) {
                if ( !$Day->isEmpty() ) {
                    $this->day = $Day->thisDay();
                    break;
                }
            }
        }
    }
}
?>